// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class EnvelopeFilter extends NonCausal implements SISOFilter {
    
    private int M = 100;
    private QuadHybrid ipf = new QuadHybrid(), qpf = new QuadHybrid();
    private double[] pipf = new double[] {0, M}, pqpf = new double[] {-90, M};

    protected double iSamp(float[] src, int n) {return ipf.outputSamp(src, n);}
    
    protected double qSamp(float[] src, int n) {return qpf.outputSamp(src, n);}

    protected void Init(double sample_ms) {
        super.Init(sample_ms);
        setM_pre(M); setM_pst(M);
        pipf[1] = M; ipf.setParams(pipf); ipf.setMs(sample_ms); 
        pqpf[1] = M; qpf.setParams(pqpf); qpf.setMs(sample_ms);
    }
		
    protected void Apply(float[] src, float[] dst) {
        for (int n = 0; n < dst.length; n++) {
        	double si = iSamp(src, n), sq = qSamp(src, n);
            double ssq = si*si + sq*sq;
            if (ssq > 0.0)
                dst[n] = (float) Math.sqrt(ssq);
            else
                dst[n] = 0.0f;
        }
    }

    public void setParams(double[] params) {
        if (params.length > 0) M = (int) params[0];
    }
}