// Copyright 2011 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.
// Free software: Permission granted to use, copy, excerpt or share this
// source code and derived executables provided that this notice is maintained.  

import net.cspeech.DSP.*;

public class LoopTest extends InPlace implements SISOFilter {
    
// Copy the following two methods into ModDemod
//
//    public void upsample() {
//        WaveSourceJ src = getWaveSourceJ(1);
//        src.resample(1, 2); 
//        src.resample(1, 8);
//        setWaveSourceJ(1, src);    	
//    }
//    
//    public void loopTest() {
//        WaveSourceJ src = getWaveSourceJ(1);
//        /* params[0 .. 3] ignored but set same as for FMLoopC	
//        src.javaFilt("LoopTest", new double[] {88200, 1.0, 40000.0, 0, 16000.0, 1.});
//        setWaveSourceJ(1, src);    	
//    }
//
// Connect the two methods to the Filter menu by copying these two methods to TF32JMod
//
//    public void upsample() {ClassLoad.invoke("ModDemod", "upsample", this);}
//    
//    public void loopTest() {ClassLoad.invoke("ModDemod",  "loopTest", this);}
//    
// Finally, add these two function calls to the TF32JMod.extendMenu() method
//	
//    Filter_menu.add(menuItem("Upsample", "upsample"));
//    Filter_menu.add(menuItem("Loop test", "loopTest"));

//   With these new methods,
//   1.  Zero Ch 1 and upsample
//   2.  From the Filter menu, apply an impulse train to upsampled Ch 1
//   3.  Apply loopTest, and select an impulse response,
//       unchecking Hamming window and Preemphasis in the time-slice spectrum	
//
//   The above selection of f_loop = 16000 Hz and damping rho = 1 gives a wide frequency
//   closed-loop frequency response for the PLL, but this gave an improvement over
//   "direct FM" in immunity to clicks in the PLL FM demodulator.	
   
    private double f_loop = 12000, rho = 1., y1;
    private LP1st loop_filt = new LP1st();

    protected void Init(double sample_ms) {
        super.Init(sample_ms);

        double T = 0.001*sample_ms;
        double theta = Math.sqrt(2.0)*Math.PI*f_loop*T, gamma = 1.0 - rho*theta,
               gamma2 = gamma*gamma,
               d0 = 1. - 2.*gamma*Math.cos(theta) + gamma2,
               b1 = 1 - gamma2 - 0*d0,
               b0 = b1 + d0,
               a1 = gamma2 + b1;
        y1 = 0;
        
        System.out.println(b0); System.out.println(b1);
        System.out.println(theta); System.out.println(gamma);
        loop_filt.setParams(new double[] {a1, b0, b1});
        loop_filt.setMs(sample_ms);
    }
		
    protected void Apply(float[] src) {
        for (int n = 0; n < src.length; n++) {
           
            y1 += loop_filt.gety0(src[n] - y1);
            src[n] = (float) y1;
        }
    }

    public void setParams(double[] params) {
        if (params != null) {
         // params[0 .. 3] ignored 
            if (params.length > 4) f_loop = params[4];
            if (params.length > 5) rho = params[5];
        }
    }
}
