﻿// Copyright 2020 Paul Milenkovic

public class ModDemod {
	private TF32J tf32j;
 // Predefined parameter arrays for filter modules
	private double[] modI  = new double[] {20000.0, 1.0,   0.0},
	                 modQ  = new double[] {20000.0, 1.0, -90.0},
	                 modI2 = new double[] {20000.0, 2.0,   0.0},
	                 modQ2 = new double[] {20000.0, 2.0,  90.0},
	                 modf1 = new double[] {11000.0, 1.0,   0.0},
	       	    	 modf2 = new double[] {33000.0, 1.0,   0.0},
	       	    	 modf12= new double[] {11000.0, 2.0,   0.0},
	       	    	 modf22= new double[] {33000.0, 2.0,   0.0},
	                 hybI  = new double[] {0.0}, hybQ = new double[] {-90.0};
    	
	public ModDemod(TF32J tf32j) {this.tf32j = tf32j;}
	
	private WaveSourceJ getWaveSourceJ(int ch) {return tf32j.getWaveSourceJ(ch);}
	
	private void setWaveSourceJ(int ch, WaveSourceJ src) {tf32j.setWaveSourceJ(ch, src);}
	
    public void modSSB() {      	
    	WaveSourceJ srcI = getWaveSourceJ(1);
    	srcI.javaFilt("QuadHybrid", hybI);
    	srcI.resample(1, 4);
    	srcI.javaFilt("SineModulator", modI);
    	
    	WaveSourceJ srcQ = getWaveSourceJ(1);
    	srcQ.javaFilt("QuadHybrid", hybQ);
    	srcQ.resample(1, 4);
    	srcQ.javaFilt("SineModulator", modQ);
    	
    	srcI.sub(srcQ);	// for LSB, use add(
    	setWaveSourceJ(2, srcI);    	
    }

    public void demodSSB() {    	   	
    	WaveSourceJ srcI = getWaveSourceJ(2);
    	srcI.javaFilt("SineModulator", modI);
    	srcI.resample(4, 1);
    	srcI.javaFilt("QuadHybrid", hybI);
    	
    	WaveSourceJ srcQ = getWaveSourceJ(2);
    	srcQ.javaFilt("SineModulator", modQ);
    	srcQ.resample(4, 1);
    	srcQ.javaFilt("QuadHybrid", hybQ);
    
    	srcI.add(srcQ);	// for LSB, use sub(
    	setWaveSourceJ(3, srcI);
    }

    private void modDSB(double[] mod_ch1, double[] mod_ch2) {    	
    	WaveSourceJ srcI = getWaveSourceJ(1);
    	srcI.resample(1, 4);
    	srcI.javaFilt("SineModulator", mod_ch1);
    	
    	WaveSourceJ srcQ = getWaveSourceJ(2);
    	srcQ.resample(1, 4);
    	srcQ.javaFilt("SineModulator", mod_ch2);
    	
    	srcI.sub(srcQ); setWaveSourceJ(3, srcI);
    }

    private void demodDSB(double[] demod_ch1, double[] demod_ch2) {       
        WaveSourceJ src = getWaveSourceJ(3);
        src.javaFilt("SineModulator", demod_ch1);
        src.resample(4, 1);
        WaveSourceJ ch4 = new WaveSourceJ(src); // fan out
        setWaveSourceJ(4, src);
        
        WaveSourceJ src2 = getWaveSourceJ(3);
        src2.javaFilt("SineModulator", demod_ch2);
        src2.resample(4, 1);
        setWaveSourceJ(5, src2);
        
        WaveSourceJ ch1 = getWaveSourceJ(1);
        ch1.sub(ch4);
        setWaveSourceJ(6, ch1);
    }  
    
    public void modDSB() {modDSB(modI, modQ);}
    
    public void demodDSB() {demodDSB(modI2, modQ2);}
   
    private double[] FM_params = new double[] {88200, 1.0, 40000.0, .1, 16000, 1};
    // {fc, ac, Kf, noise RMS (channel), f_l of loop filter, damping of loop filter (.5 less damping, 2 more damping)}

    public void modFM() {
        WaveSourceJ src = getWaveSourceJ(1);
        src.resample(1, 2);
        src.resample(1, 8);
        src.javaFilt("FMModulator", FM_params);
        setWaveSourceJ(2, src);
    }
    
    public void demodFM() {
    	WaveSourceJ src = getWaveSourceJ(2);
    	src.javaFilt("FMDemodulator", FM_params);
    	src.resample(8, 1);
    	src.resample(2, 1);
    	WaveSourceJ nsrc = new WaveSourceJ(src); // Provides fan-out
    	setWaveSourceJ(3, src);
     // This method of measuring received noise only valid for 
     // the accurate "direct phase" demodulator	
    	WaveSourceJ original = getWaveSourceJ(1);
    	nsrc.sub(original);
        setWaveSourceJ(4, nsrc);
    }

    public void demodFMPLL() {
    	WaveSourceJ src = getWaveSourceJ(2);
    	src.javaFilt("FMLoopC", FM_params);
    	src.resample(8, 1);
    	src.resample(2, 1);
     // PLL introduces linear frequency-distortion, hence use of the notch
     // filter applicable to a message waveform of a single 1000 Hz frequency	
    	WaveSourceJ nsrc = new WaveSourceJ(src);
    	nsrc.vocFilt("FIRNotchI", new double[] {400, 1000.0});
    	setWaveSourceJ(3, src);
    	setWaveSourceJ(4, nsrc);
    }
    
    public void upsample() {
        WaveSourceJ src = getWaveSourceJ(1);
        src.resample(1, 2); 
        src.resample(1, 8);
        setWaveSourceJ(1, src);    	
    }
    
    public void loopTest() {
        WaveSourceJ src = getWaveSourceJ(1);
        src.javaFilt("LoopTest", FM_params);
        setWaveSourceJ(1, src);    	
    }
}
