// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef PulseSynH
#define PulseSynH

#include <math.h>
#include "LPCObj.h"
#include "AddTrack.h"

class PulseSyn : public AddTrack 
{
private:
    double sample_ms;

    bool valid_epoch;
    double t_epoch;
    int i_epoch;

    double glot_amp;

    bool GetEpoch()
    {
        bool result = GetLine() && GetNum(t_epoch);
        if (result)
            i_epoch = int(t_epoch/sample_ms);
        return result;
    }

    bool GetParams()
    {
        bool result = GetNum(glot_amp);
        return result;
    }

    void PutPulse(float* sig_buf, int ipulse)
    {
        sig_buf[ipulse] = (float) (-glot_amp);
    }

protected:
    int Get_mem() {return 4096;}

    void Init(IWaveDescriptorC* dsc)
    {
        SISOFilter::Init(dsc);
    }

    void Init(double sample_ms)
    {
        this->sample_ms = sample_ms;
        valid_epoch = Open(".gpm") && GetEpoch();
    }

    void Apply(float sig_buf[], int nbuf, int npos)
    {
        while (valid_epoch && i_epoch < npos + nbuf - 1024) 
        {
            if (GetParams()) {
                if (i_epoch-npos >= 0)
                    PutPulse(sig_buf,i_epoch-npos);
                valid_epoch = GetEpoch();
            } else
                valid_epoch = false;
        }
    }
};

#endif
