// Copyright 2009 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef SetTrackH
#define SetTrackH

#include <string>
#include "SISOFilter.h"
#include "NumStream.h"
#include "WaveCpp.h"

class SetTrack : public SISOFilter 
{
private:
    NumStream num_stream;
	bool end_of_src;
	int mbuf, nbuf, ibuf, nch, sch, hdr_bytes, msamp;
	double sample_ms;
    short buf[4096];

protected:
    string SetExt(string fname, string ext)
    {
        size_t epos = fname.rfind('.');
        if (epos != string::npos)
            fname = fname.substr(0,epos);
        return fname + ext;    
    }

	bool Open(string fname) 
    {
        return num_stream.Open((char *) fname.c_str());
    }

    void Close() {num_stream.Close();}

	virtual void Init(IWaveDescriptorC* dsc)
    {
		SISOFilter::Init(dsc);
		end_of_src = !Open(SetExt(dsc->Get_fname(),".iop"));
		sample_ms = 1000.0/873.3;
		nch = 1;
		sch = 1;
		hdr_bytes = 1024;
		mbuf = nch*(4096/nch);
		nbuf = 0;
		ibuf = 0;
		num_stream.SkipBytes(hdr_bytes);
		msamp = num_stream.BytesRemain()/(2*nch);
		end_of_src = end_of_src | (msamp < 1);
    }


public:
    virtual bool STDCALL EndOfSrc()
    {
        return end_of_src;
    }

    virtual void  STDCALL GetDescriptor(IWaveDescriptorC** dsc) 
    {
		SISOFilter::GetDescriptor(dsc);
		(*dsc)->Set_sample_ms(sample_ms);
		(*dsc)->Set_msamp(msamp);
    }

    void  STDCALL TransmitNow(float* wave_samp, int nBuf, int& nSamp)
    {
		nSamp = 0;
		while (!end_of_src && nSamp < nBuf) {

		    int n = (nbuf - ibuf)/nch;
			if (n > nBuf - nSamp) n = nBuf - nSamp;
			for (int i = 0; i < n; i++)
				wave_samp[nSamp+i] = (float) (buf[ibuf+(sch-1)+i*nch]*10.0/32768);
			ibuf += n*nch;
			nSamp += n;

			if (ibuf >= nbuf) {
				ibuf = 0;
				nbuf = nch*(num_stream.BytesRemain()/(2*nch));
			    end_of_src = nbuf < nch;
			    if (!end_of_src) {
				    if (nbuf > mbuf) nbuf = mbuf;
					char* bufb = (char *) buf;
					num_stream.ReadBytes(bufb,nbuf*2);
					for (int i = 0; i < nbuf; i++) {
						char temp = bufb[2*i];
                        bufb[2*i] = bufb[2*i+1];
                        bufb[2*i+1] = temp;
					}
			    }
			}
		}
    }

	virtual void  STDCALL Dispose() 
    {
        num_stream.Close();
        SISOFilter::Dispose();
    }

    SetTrack() : num_stream(true), end_of_src(true) {}

    virtual ULONG STDCALL Release() 
    {
        int rc = decRefCount();
        if (rc <= 0) delete this;
        return rc;
    }
};

#endif
