// Copyright 2007 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef SquareFilterH
#define SquareFilterH

#include "InPlace.h"
#include <math.h>

class SquareFilter : public InPlace
{
private:
    int nseq;
    double b0;
    double b1;
    double x0, x1;
    int iseq;

protected:
    virtual void Init(double sample_ms)
    {
       x0 = -1.0;
       x1 = -1.0;   
       iseq = 0;    
    }

    virtual void Apply(float* sig, int nSig)
    {
        for (int i = 0; i < nSig; i++) {

            sig[i] = (float) (b0*x0 + b1*x1);
            x1 = x0;
            iseq++;
            if (iseq >= nseq) {
            
                x0 = -x0;
                iseq = 0;
            }
        }
    }

public:
    SquareFilter() : 
        nseq(256), b0(1.0), b1(0.0) 
    {}

    SquareFilter(int nseq, double b0, double b1) :
        nseq(256), b0(1.0), b1(0.0) 
    {}

    virtual void  STDCALL Set_param_list(double* param_list, int nParam) 
    {
        if (nParam > 0) nseq = (int) param_list[0];
        if (nParam > 1) b0 = param_list[1];
        if (nParam > 2) b1 = param_list[2];
    }
};

#endif
