// Copyright 2004 Paul Milenkovic, 118 Shiloh Dr., Madison, WI 53705 U.S.A.

#ifndef VocFiltH
#define VocFiltH

// Need COMBase.h here for COM types and consts --
// include <windows.h> instead for a Windows DLL.
#ifdef _WIN32
#include <windows.h>
#define STDCALL __stdcall
#define LIBEXPORT extern "C" __declspec(dllexport)
#else
#include "COMBase.h"
#endif

#include "WaveCpp.h"
#include "WaveBase.h"
#include "FilterFactory.h"

#include "SetTrack.h"
#include "BeamPel.h"
#include "BeamData.h"
#include "GlotSyn.h"
#include "FricSyn.h"
#include "PulseSyn.h"
#include "FIRFilter.h"
#include "FIRNotch.h"
#include "ResampleBlackman.h"
#include "SumSources.h"
#include "EchoCancel.h"
#include "SineFilter.h"
#include "SquareFilter.h"

class SetTrackFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new SetTrack();
		(*wave_filter)->AddRef();
		return true;
	}
};

class BeamPelFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new BeamPel();
		(*wave_filter)->AddRef();
		return true;
	}
};

class BeamDataFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new BeamData();
		(*wave_filter)->AddRef();
		return true;
	}
};

class GlotSynFactory : public FilterFactory
{
public:
    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = new GlotSyn();
        (*wave_filter)->AddRef();
        return true;
    }
};

class FricSynFactory : public FilterFactory
{
public:
    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = new FricSyn();
        (*wave_filter)->AddRef();
        return true;
    }
};

class PulseSynFactory : public FilterFactory
{
public:
    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = new PulseSyn();
        (*wave_filter)->AddRef();
        return true;
    }
};

class FIRFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new FIRFilter();
		(*wave_filter)->AddRef();
		return true;
	}
};

class FIRNotchFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new FIRNotch();
		(*wave_filter)->AddRef();
		return true;
	}
};

class ResampleFactory : public FilterFactory
{
public:
    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = new ResampleBlackman();
        (*wave_filter)->AddRef();
        return true;
    }
};

class SumSourcesFactory : public FilterFactory
{
public:
    virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
    {
        *wave_filter = new NonCausalP(new SumSources());
        (*wave_filter)->AddRef();
        return true;
    }
};

class EchoCancelFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new NonCausalP(new EchoCancel());
		(*wave_filter)->AddRef();
		return true;
	}
};

class SineFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new SineFilter();
		(*wave_filter)->AddRef();
		return true;
	}
};

class SquareFactory : public FilterFactory
{
public:
	virtual bool STDCALL GetFilter(IWaveFilterC** wave_filter)
	{
		*wave_filter = new SquareFilter();
		(*wave_filter)->AddRef();
		return true;
	}
};

#endif
